#include <gint/display.h>
#include <gint/drivers/r61524.h>

#include <gint/keyboard.h>
#include <gint/gint.h>
#include <gint/defs/types.h>
#include <fxlibc/printf.h>

#include <libprof.h>
#include <gint/usb.h>
#include <gint/usb-ff-bulk.h>

#include <math.h>
#include <vector>


#include "include/camera.h"
#include "include/segment.h"
#include "parameters.h"
#include "include/circuit.h"


//extern bopti_image_t car1, car2, car3, car4, car5, car6, car7, car8;


std::vector<Segment*> circuit;
camera *cam;

bool stop = false;
bool record = false;
bool screenshot = false;

uint16_t currentcurve=0;
uint8_t shiftcolor=0;

float speed = 0;
float maxspeedforward = 50;
float maxspeedbackward = 25;
int direction = 1;
bool speedcontrol = false;


static int get_inputs( float dt )
{
    key_event_t ev;
    while((ev = pollevent()).type != KEYEV_NONE)
    {

    }

    speedcontrol = false;

    if(keydown(KEY_LEFT))  cam->cX-=fix(25);
    if(keydown(KEY_RIGHT)) cam->cX+=fix(25);

    if(keydown(KEY_SHIFT))
    {
        if (direction==-1 && speed > 0)
        {
            direction = -1;
            speed -= 5;
            if (speed<0) speed=0;
        }
        else
        {
            direction = 1;
            speed+=0.5;
            if (speed>maxspeedforward) speed=maxspeedforward;
        }
        cam->cZ+=speed*direction*dt;
        speedcontrol = true;
    }
    if(keydown(KEY_ALPHA))
    {
        if (direction==1 && speed > 0)
        {
            direction = 1;
            speed -= 5;
            if (speed<0) speed=0;
        }
        else
        {
            direction = -1;
            speed+=0.25;
            if (speed>maxspeedbackward) speed=maxspeedbackward;
        }
        cam->cZ+=speed*direction*dt;
        speedcontrol = true;
    }
    if(keydown(KEY_7))     cam->cY+=10;
    if(keydown(KEY_1))     cam->cY-=10;
    if(keydown(KEY_MENU))  gint_osmenu();
    if(keydown(KEY_EXIT))  stop = true;
    if(keydown(KEY_F6))    record = !record;
    if(keydown(KEY_F4))    screenshot=true;

    if (speedcontrol==false)
    {
        speed-=3;
        if (speed<0) speed=0;
        cam->cZ+=speed*direction*dt;
    }
}


int main(void)
{
    __printf_enable_fp();
    __printf_enable_fixed();

//    usb_interface_t const *interfaces[] = { &usb_ff_bulk, NULL };
//    usb_open(interfaces, GINT_CALL_NULL);
    prof_t perf_create, perf_project, perf_render;
    uint32_t time_create=0, time_project=0, time_render=0;

    uint8_t nbInterestingSegments = (MAX_RENDER_DISTANCE / SEGMENT_LENGTH) + 10;




    initData( );

    prof_init();


    perf_create = prof_make();
    prof_enter(perf_create);

    createCircuit();

    prof_leave(perf_create);
    time_create = prof_time(perf_create);


    while (!stop)
    {
        //--------------

        get_inputs( (time_render+time_project)/10000 );

        //if (fround(cam->cZ)<0) cam->cZ=fix(0);
        //if (fround(cam->cZ)>950*SEGMENT_LENGTH) cam->cZ=fix(950*SEGMENT_LENGTH);

        int indexstart = (int) ((fround(cam->cZ)-400)/SEGMENT_LENGTH);

        //--------------

        perf_project = prof_make();
        prof_enter(perf_project);

            for (int k=indexstart; k<indexstart+nbInterestingSegments+1; k++)       // Need to project 1 more segment than actual drawing
                projectCircuitFP( k );

        prof_leave(perf_project);
        time_project = prof_time(perf_project);

        //--------------

        perf_render = prof_make();
        prof_enter(perf_render);

            drawSky();

            for (int k=indexstart; k<indexstart+nbInterestingSegments; k++)
                drawCircuitSegment( k );

            r61524_display(gint_vram, 0, DHEIGHT, R61524_DMA_WAIT);

        prof_leave(perf_render);
        time_render = prof_time(perf_render);

        dprint( 1, 1, C_RED, "Circuit Creation = %.3D ms", time_create );
        dprint( 1, 15, C_RED, "Segments Projection = %.3D ms", time_project );
        dprint( 1, 29, C_RED, "Rendering = %.3D ms", time_render );

        dupdate();

        //if (screenshot) usb_fxlink_screenshot(true);

    }

    prof_quit();
    //usb_close();

    circuit.clear();
    delete cam;

    return 1;
}
